@@ -24,71 +24,88 @@
 Module that implements the gramplet bar fuctionality.
 """
 
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 #
 # Set up logging
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 import logging
-LOG = logging.getLogger('.grampletbar')
 
-#-------------------------------------------------------------------------
+LOG = logging.getLogger(".grampletbar")
+
+# -------------------------------------------------------------------------
 #
 # Python modules
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 import time
 import os
 import configparser
 
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 #
 # GNOME modules
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 from gi.repository import Gtk
+from gi.repository import Gio
+from gi.repository import GLib
 
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 #
 # Gramps modules
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 from gramps.gen.const import GRAMPS_LOCALE as glocale
+
 _ = glocale.translation.gettext
 from gramps.gen.const import URL_MANUAL_PAGE, VERSION_DIR
 from gramps.gen.config import config
 from gramps.gen.constfunc import win
+from gramps.gen.utils.configmanager import clean_up
 from ..managedwindow import ManagedWindow
 from ..display import display_help, display_url
-from .grampletpane import (AVAILABLE_GRAMPLETS,
-                           GET_AVAILABLE_GRAMPLETS,
-                           GET_GRAMPLET_LIST,
-                           get_gramplet_opts,
-                           get_gramplet_options_by_name,
-                           make_requested_gramplet,
-                           GuiGramplet)
+from .grampletpane import (
+    AVAILABLE_GRAMPLETS,
+    GET_AVAILABLE_GRAMPLETS,
+    GET_GRAMPLET_LIST,
+    get_gramplet_opts,
+    get_gramplet_options_by_name,
+    make_requested_gramplet,
+    GuiGramplet,
+)
 from .undoablebuffer import UndoableBuffer
-from ..utils import is_right_click
 from ..dialog import QuestionDialog
 
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 #
 # Constants
 #
-#-------------------------------------------------------------------------
-WIKI_HELP_PAGE = URL_MANUAL_PAGE + '_-_Gramplets'
+# -------------------------------------------------------------------------
+WIKI_HELP_PAGE = URL_MANUAL_PAGE + "_-_Gramplets"
+MAIN_HELP_PAGE = URL_MANUAL_PAGE + "_-_Main_Window"
 NL = "\n"
 
-#-------------------------------------------------------------------------
+
+# -------------------------------------------------------------------------
 #
 # GrampletBar class
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 class GrampletBar(Gtk.Notebook):
     """
     A class which defines the graphical representation of the GrampletBar.
     """
-    def __init__(self, dbstate, uistate, pageview, configfile, defaults):
+
+    def __init__(
+        self,
+        dbstate,
+        uistate,
+        pageview,
+        configfile,
+        defaults,
+        orientation=Gtk.Orientation.VERTICAL,
+    ):
         Gtk.Notebook.__init__(self)
 
         self.dbstate = dbstate
@@ -96,6 +113,7 @@ class GrampletBar(Gtk.Notebook):
         self.pageview = pageview
         self.configfile = os.path.join(VERSION_DIR, "%s.ini" % configfile)
         self.defaults = defaults
+        self.orientation = orientation
         self.detached_gramplets = []
         self.empty = False
         self.close_buttons = []
@@ -104,24 +122,33 @@ class GrampletBar(Gtk.Notebook):
         self.set_show_border(False)
         self.set_scrollable(True)
 
-        book_button = Gtk.Button()
-        # Arrow is too small unless in a box
-        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        arrow = Gtk.Arrow(arrow_type=Gtk.ArrowType.DOWN,
-                                    shadow_type=Gtk.ShadowType.NONE)
-        arrow.show()
-        box.add(arrow)
-        box.show()
-        book_button.add(box)
-        book_button.set_relief(Gtk.ReliefStyle.NONE)
-        book_button.connect('clicked', self.__button_clicked)
-        book_button.set_property("tooltip-text", _("Gramplet Bar Menu"))
-        book_button.show()
-        self.set_action_widget(book_button, Gtk.PackType.END)
+        menu_button = Gtk.MenuButton(
+            direction=Gtk.ArrowType.DOWN,
+            relief=Gtk.ReliefStyle.NONE,
+            tooltip_text=_("Gramplet Bar Menu"),
+            use_popover=False,
+        )
+        menu_button.show()
+        self.set_action_widget(menu_button, Gtk.PackType.END)
+        action_group = Gio.SimpleActionGroup()
+        action_group.add_action_entries(
+            [
+                ("add", self.__add_clicked, "s"),
+                ("remove", self.__remove_clicked, "s"),
+                ("restore", self.__restore_clicked),
+                ("help", self.on_help_grampletbar_clicked),
+                ("about", self.on_help_gramplets_clicked),
+            ],
+            None,
+        )
+        menu_button.insert_action_group("menu", action_group)
+        self.menu = Gio.Menu()
+        menu_button.set_menu_model(self.menu)
+        self.__refresh_menu()
 
-        self.connect('page-added', self.__page_added)
-        self.connect('page-removed', self.__page_removed)
-        self.connect('create-window', self.__create_window)
+        self.connect("page-added", self.__page_added)
+        self.connect("page-removed", self.__page_removed)
+        self.connect("create-window", self.__create_window)
 
         config_settings, opts_list = self.__load(defaults)
 
@@ -129,8 +156,9 @@ class GrampletBar(Gtk.Notebook):
         for opts in opts_list:
             if opts["name"] in AVAILABLE_GRAMPLETS():
                 all_opts = get_gramplet_opts(opts["name"], opts)
-                gramplet = make_requested_gramplet(TabGramplet, self, all_opts,
-                                                   self.dbstate, self.uistate)
+                gramplet = make_requested_gramplet(
+                    TabGramplet, self, all_opts, self.dbstate, self.uistate
+                )
                 if gramplet:
                     self.__add_tab(gramplet)
 
@@ -142,10 +170,10 @@ class GrampletBar(Gtk.Notebook):
             self.show()
         self.set_current_page(config_settings[1])
 
-        uistate.connect('grampletbar-close-changed', self.cb_close_changed)
+        uistate.connect("grampletbar-close-changed", self.cb_close_changed)
 
         # Connect after gramplets added to prevent making them active
-        self.connect('switch-page', self.__switch_page)
+        self.connect("switch-page", self.__switch_page)
 
     def _get_config_setting(self, configparser, section, setting, fn=None):
         """
@@ -179,7 +207,7 @@ class GrampletBar(Gtk.Notebook):
         if filename and os.path.exists(filename):
             cp = configparser.ConfigParser()
             try:
-                cp.read(filename, encoding='utf-8')
+                cp.read(filename, encoding="utf-8")
             except:
                 pass
             for sec in cp.sections():
@@ -193,19 +221,21 @@ class GrampletBar(Gtk.Notebook):
                     for opt in cp.options(sec):
                         if opt.startswith("data["):
                             temp = data.get("data", {})
-                            #temp.append(self._get_config_setting(cp, sec, opt))
+                            # temp.append(self._get_config_setting(cp, sec, opt))
                             pos = int(opt[5:-1])
                             temp[pos] = self._get_config_setting(cp, sec, opt)
                             data["data"] = temp
                         else:
                             data[opt] = self._get_config_setting(cp, sec, opt)
                     if "data" in data:
-                        data["data"] = [data["data"][key]
-                                        for key in sorted(data["data"].keys())]
+                        data["data"] = [
+                            data["data"][key] for key in sorted(data["data"].keys())
+                        ]
                     if "name" not in data:
                         data["name"] = "Unnamed Gramplet"
                         data["tname"] = _("Unnamed Gramplet")
                     retval.append(data)
+            clean_up(cp)
         else:
             # give defaults as currently known
             for name in defaults:
@@ -219,20 +249,28 @@ class GrampletBar(Gtk.Notebook):
         """
         filename = self.configfile
         try:
-            with open(filename, "w", encoding='utf-8') as fp:
+            with open(filename, "w", encoding="utf-8") as fp:
                 fp.write(";; Gramplet bar configuration file" + NL)
-                fp.write((";; Automatically created at %s" %
-                                         time.strftime("%Y/%m/%d %H:%M:%S")) + NL + NL)
+                fp.write(
+                    (
+                        ";; Automatically created at %s"
+                        % time.strftime("%Y/%m/%d %H:%M:%S")
+                    )
+                    + NL
+                    + NL
+                )
                 fp.write("[Bar Options]" + NL)
-                fp.write(("visible=%s" + NL) % self.get_property('visible'))
+                fp.write(("visible=%s" + NL) % self.get_property("visible"))
                 fp.write(("page=%d" + NL) % self.get_current_page())
                 fp.write(NL)
 
                 if self.empty:
                     gramplet_list = []
                 else:
-                    gramplet_list = [self.get_nth_page(page_num)
-                                     for page_num in range(self.get_n_pages())]
+                    gramplet_list = [
+                        self.get_nth_page(page_num)
+                        for page_num in range(self.get_n_pages())
+                    ]
 
                 for page_num, gramplet in enumerate(gramplet_list):
                     opts = get_gramplet_options_by_name(gramplet.gname)
@@ -243,8 +281,16 @@ class GrampletBar(Gtk.Notebook):
                                 base_opts[key] = gramplet.__dict__[key]
                         fp.write(("[%s]" + NL) % gramplet.gname)
                         for key in base_opts:
-                            if key in ["content", "title", "tname", "row", "column",
-                                       "page", "version", "gramps"]: # don't save
+                            if key in [
+                                "content",
+                                "title",
+                                "tname",
+                                "row",
+                                "column",
+                                "page",
+                                "version",
+                                "gramps",
+                            ]:  # don't save
                                 continue
                             elif key == "data":
                                 if not isinstance(base_opts["data"], (list, tuple)):
@@ -255,7 +301,7 @@ class GrampletBar(Gtk.Notebook):
                                         fp.write(("data[%d]=%s" + NL) % (cnt, item))
                                         cnt += 1
                             else:
-                                fp.write(("%s=%s" + NL)% (key, base_opts[key]))
+                                fp.write(("%s=%s" + NL) % (key, base_opts[key]))
                         fp.write(("page=%d" + NL) % page_num)
                         fp.write(NL)
 
@@ -303,8 +349,9 @@ class GrampletBar(Gtk.Notebook):
         if self.has_gramplet(gname):
             return
         all_opts = get_gramplet_options_by_name(gname)
-        gramplet = make_requested_gramplet(TabGramplet, self, all_opts,
-                                           self.dbstate, self.uistate)
+        gramplet = make_requested_gramplet(
+            TabGramplet, self, all_opts, self.dbstate, self.uistate
+        )
         if not gramplet:
             LOG.warning("Problem creating '%s'", gname)
             return
@@ -341,8 +388,10 @@ class GrampletBar(Gtk.Notebook):
         if self.empty:
             return self.detached_gramplets
         else:
-            return [gramplet.gname for gramplet in self.get_children() +
-                                                   self.detached_gramplets]
+            return [
+                gramplet.gname
+                for gramplet in self.get_children() + self.detached_gramplets
+            ]
 
     def restore(self):
         """
@@ -356,9 +405,11 @@ class GrampletBar(Gtk.Notebook):
         """
         Create an empty tab to be displayed when the GrampletBar is empty.
         """
-        tab_label = Gtk.Label(label=_('Gramplet Bar'))
+        tab_label = Gtk.Label(label=_("Gramplet Bar"))
         tab_label.show()
-        msg = _('Select the down arrow on the right corner for adding, removing or restoring gramplets.')
+        msg = _(
+            "Select the down arrow on the right corner for adding, removing or restoring gramplets."
+        )
         content = Gtk.Label(label=msg)
         content.set_halign(Gtk.Align.START)
         content.set_line_wrap(True)
@@ -383,10 +434,10 @@ class GrampletBar(Gtk.Notebook):
 
         if hasattr(gramplet.pui, "has_data"):
             tablabel.set_has_data(gramplet.pui.has_data)
-        else: # just a function; always show yes it has data
+        else:  # just a function; always show yes it has data
             tablabel.set_has_data(True)
 
-        if config.get('interface.grampletbar-close'):
+        if config.get("interface.grampletbar-close"):
             tablabel.use_close(True)
         else:
             tablabel.use_close(False)
@@ -400,7 +451,7 @@ class GrampletBar(Gtk.Notebook):
         for gramplet in self.get_children():
             tablabel = self.get_tab_label(gramplet)
             if not isinstance(tablabel, Gtk.Label):
-                tablabel.use_close(config.get('interface.grampletbar-close'))
+                tablabel.use_close(config.get("interface.grampletbar-close"))
 
     def __delete_clicked(self, button, gramplet):
         """
@@ -431,6 +482,8 @@ class GrampletBar(Gtk.Notebook):
         Called when a new page is added to the GrampletBar.
         """
         gramplet = self.get_nth_page(new_page)
+        if isinstance(gramplet, TabGramplet):
+            gramplet.set_orientation(self.orientation)
         if self.empty:
             if isinstance(gramplet, TabGramplet):
                 self.empty = False
@@ -448,6 +501,7 @@ class GrampletBar(Gtk.Notebook):
         if gramplet in self.detached_gramplets:
             self.detached_gramplets.remove(gramplet)
             self.reorder_child(gramplet, gramplet.page)
+        self.__refresh_menu()
 
     def __page_removed(self, notebook, unused, page_num):
         """
@@ -456,6 +510,7 @@ class GrampletBar(Gtk.Notebook):
         if self.get_n_pages() == 0:
             self.empty = True
             self.__create_empty_tab()
+        self.__refresh_menu()
 
     def __create_window(self, grampletbar, gramplet, x_pos, y_pos):
         """
@@ -474,78 +529,78 @@ class GrampletBar(Gtk.Notebook):
         gramplet.detached_window.close()
         gramplet.detached_window = None
 
-    def __button_clicked(self, button):
+    def __refresh_menu(self):
         """
         Called when the drop-down button is clicked.
         """
-        self.menu = Gtk.Menu()
         menu = self.menu
+        menu.remove_all()
 
-        ag_menu = Gtk.MenuItem(label=_('Add a gramplet'))
         nav_type = self.pageview.navigation_type()
         skip = self.all_gramplets()
         gramplet_list = GET_GRAMPLET_LIST(nav_type, skip)
         gramplet_list.sort()
-        self.__create_submenu(ag_menu, gramplet_list, self.__add_clicked)
-        ag_menu.show()
-        menu.append(ag_menu)
+        ag_menu = self.__create_submenu(gramplet_list, "add")
+        menu.append_submenu(label=_("Add a gramplet"), submenu=ag_menu)
 
-        if not (self.empty or config.get('interface.grampletbar-close')):
-            rg_menu = Gtk.MenuItem(label=_('Remove a gramplet'))
-            gramplet_list = [(gramplet.title, gramplet.gname)
-                             for gramplet in self.get_children() +
-                                             self.detached_gramplets]
+        if not (self.empty or config.get("interface.grampletbar-close")):
+            gramplet_list = [
+                (gramplet.title, gramplet.gname)
+                for gramplet in self.get_children() + self.detached_gramplets
+            ]
             gramplet_list.sort()
-            self.__create_submenu(rg_menu, gramplet_list,
-                                  self.__remove_clicked)
-            rg_menu.show()
-            menu.append(rg_menu)
+            rg_menu = self.__create_submenu(gramplet_list, "remove")
+            menu.append_submenu(label=_("Remove a gramplet"), submenu=rg_menu)
 
-        rd_menu = Gtk.MenuItem(label=_('Restore default gramplets'))
-        rd_menu.connect("activate", self.__restore_clicked)
-        rd_menu.show()
-        menu.append(rd_menu)
+        menu.append(
+            label=_("Restore default gramplets"), detailed_action="menu.restore"
+        )
 
-        menu.show_all()
-        menu.popup(None, None, cb_menu_position, button, 0, 0)
+        section = Gio.Menu()
+        section.append(label=_("Gramplet Bar Help"), detailed_action="menu.help")
+        section.append(label=_("About Gramplets"), detailed_action="menu.about")
+        menu.append_section(label=None, section=section)
 
-    def __create_submenu(self, main_menu, gramplet_list, callback_func):
+    def __create_submenu(self, gramplet_list, action_name):
         """
         Create a submenu of the context menu.
         """
-        if main_menu:
-            submenu = main_menu.get_submenu()
-            submenu = Gtk.Menu()
-            for entry in gramplet_list:
-                item = Gtk.MenuItem(label=entry[0])
-                item.connect("activate", callback_func, entry[1])
-                item.show()
-                submenu.append(item)
-            main_menu.set_submenu(submenu)
+        submenu = Gio.Menu()
+        for entry in gramplet_list:
+            submenu.append(
+                label=entry[0],
+                detailed_action=f"menu.{action_name}({entry[1]!r})",
+            )
+        return submenu
 
-    def __add_clicked(self, menu, gname):
+    def __add_clicked(self, action, parameter, data):
         """
         Called when a gramplet is added from the context menu.
         """
+        gname = parameter.get_string()
         self.add_gramplet(gname)
 
-    def __remove_clicked(self, menu, gname):
+    def __remove_clicked(self, action, parameter, data):
         """
         Called when a gramplet is removed from the context menu.
         """
+        gname = parameter.get_string()
         self.remove_gramplet(gname)
 
-    def __restore_clicked(self, menu):
+    def __restore_clicked(self, action, parameter, data):
         """
         Called when restore defaults is clicked from the context menu.
         """
         QuestionDialog(
             _("Restore to defaults?"),
-            _("The gramplet bar will be restored to contain its default "
-              "gramplets.  This action cannot be undone."),
+            _(
+                "The gramplet bar will be restored to contain its default "
+                "gramplets.  This action cannot be undone."
+            ),
             _("OK"),
             self.restore,
-            parent=self.uistate.window)
+            parent=self.uistate.window,
+        )
 
     def get_config_funcs(self):
         """
@@ -567,19 +622,31 @@ class GrampletBar(Gtk.Notebook):
         Return a configuration function that returns the title of a page in
         the Configure View dialog and a gtk container defining the page.
         """
+
         def gramplet_panel(configdialog):
             return title, gui_options
+
         return gramplet_panel
 
-#-------------------------------------------------------------------------
+    def on_help_grampletbar_clicked(self, action, parameter, data):
+        """Button: Display the relevant portion of Gramps manual"""
+        display_help(MAIN_HELP_PAGE, "Bottombar_and_Sidebar")
+
+    def on_help_gramplets_clicked(self, action, parameter, data):
+        """Button: Display the relevant portion of Gramps manual"""
+        display_help(WIKI_HELP_PAGE, "What_is_a_Gramplet?")
+
+
+# -------------------------------------------------------------------------
 #
 # TabGramplet class
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 class TabGramplet(Gtk.ScrolledWindow, GuiGramplet):
     """
     Class that handles the plugin interfaces for the GrampletBar.
     """
+
     def __init__(self, pane, dbstate, uistate, title, **kwargs):
         """
         Internal constructor for GUI portion of a gramplet.
@@ -609,15 +676,24 @@ class TabGramplet(Gtk.ScrolledWindow, GuiGramplet):
         """
         return self
 
-#-------------------------------------------------------------------------
+    def set_orientation(self, orientation):
+        """
+        Called when the gramplet orientation changes.
+        """
+        if self.pui:
+            self.pui.set_orientation(orientation)
+
+
+# -------------------------------------------------------------------------
 #
 # DetachedWindow class
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 class DetachedWindow(ManagedWindow):
     """
     Class for showing a detached gramplet.
     """
+
     def __init__(self, grampletbar, gramplet, x_pos, y_pos):
         """
         Construct the window.
@@ -626,23 +702,21 @@ class DetachedWindow(ManagedWindow):
         self.grampletbar = grampletbar
         self.gramplet = gramplet
 
-        ManagedWindow.__init__(self, gramplet.uistate, [],
-                                             self.title)
-        self.set_window(Gtk.Dialog("", gramplet.uistate.window,
-                                   Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                                   (_('_Close'), Gtk.ResponseType.CLOSE)),
-                        None,
-                        self.title)
+        ManagedWindow.__init__(self, gramplet.uistate, [], self.title)
+        dlg = Gtk.Dialog(
+            transient_for=gramplet.uistate.window, destroy_with_parent=True
+        )
+        dlg.add_button(_("_Close"), Gtk.ResponseType.CLOSE)
+        self.set_window(dlg, None, self.title)
         self.window.move(x_pos, y_pos)
-        self.window.set_default_size(gramplet.detached_width,
-                                     gramplet.detached_height)
-        self.window.add_button(_('_Help'), Gtk.ResponseType.HELP)
-        self.window.connect('response', self.handle_response)
+        self.window.set_default_size(gramplet.detached_width, gramplet.detached_height)
+        self.window.add_button(_("_Help"), Gtk.ResponseType.HELP)
+        self.window.connect("response", self.handle_response)
 
         self.notebook = Gtk.Notebook()
         self.notebook.set_show_tabs(False)
         self.notebook.set_show_border(False)
-        self.notebook.connect('page-added', self.page_added)
+        self.notebook.connect("page-added", self.page_added)
         self.notebook.show()
         self.window.vbox.pack_start(self.notebook, True, True, 0)
         self.show()
@@ -663,13 +737,12 @@ class DetachedWindow(ManagedWindow):
         elif response == Gtk.ResponseType.HELP:
             # translated name:
             if self.gramplet.help_url:
-                if self.gramplet.help_url.startswith("http://"):
+                if self.gramplet.help_url.startswith(("http://", "https://")):
                     display_url(self.gramplet.help_url)
                 else:
                     display_help(self.gramplet.help_url)
             else:
-                display_help(WIKI_HELP_PAGE,
-                                   self.gramplet.tname.replace(" ", "_"))
+                display_help(WIKI_HELP_PAGE, self.gramplet.tname.replace(" ", "_"))
 
     def get_notebook(self):
         """
@@ -681,7 +754,7 @@ class DetachedWindow(ManagedWindow):
         """
         Part of the Gramps window interface.
         """
-        return (self.title, 'Gramplet')
+        return (self.title, "Gramplet")
 
     def get_title(self):
         """
@@ -697,18 +770,21 @@ class DetachedWindow(ManagedWindow):
         self.gramplet.detached_width = size[0]
         self.gramplet.detached_height = size[1]
         self.gramplet.detached_window = None
-        self.gramplet.reparent(self.grampletbar)
+        self.notebook.remove(self.gramplet)
+        self.grampletbar.add(self.gramplet)
         ManagedWindow.close(self, *args)
 
-#-------------------------------------------------------------------------
+
+# -------------------------------------------------------------------------
 #
 # TabLabel class
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 class TabLabel(Gtk.Box):
     """
     Create a tab label consisting of a label and a close button.
     """
+
     def __init__(self, gramplet, callback):
         Gtk.Box.__init__(self)
 
@@ -721,7 +797,7 @@ class TabLabel(Gtk.Box):
 
         self.closebtn = Gtk.Button()
         image = Gtk.Image()
-        image.set_from_icon_name('window-close', Gtk.IconSize.MENU)
+        image.set_from_icon_name("window-close", Gtk.IconSize.MENU)
         self.closebtn.connect("clicked", callback, gramplet)
         self.closebtn.set_image(image)
         self.closebtn.set_relief(Gtk.ReliefStyle.NONE)
@@ -747,21 +823,3 @@ class TabLabel(Gtk.Box):
             self.closebtn.show()
         else:
             self.closebtn.hide()
-
-def cb_menu_position(*args):
-    """
-    Determine the position of the popup menu.
-    """
-    # takes two argument: menu, button
-    if len(args) == 2:
-        menu = args[0]
-        button = args[1]
-    # broken introspection can't handle MenuPositionFunc annotations corectly
-    else:
-        menu = args[0]
-        button = args[3]
-    ret_val, x_pos, y_pos = button.get_window().get_origin()
-    x_pos += button.get_allocation().x
-    y_pos += button.get_allocation().y + button.get_allocation().height
-
-    return (x_pos, y_pos, False)
